/* Copyright (c) 2011, DF9DQ
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions
 * and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 * Neither the name of the author nor the names of its contributors may be used to endorse
 * or promote products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

        .syntax unified


        .text

        /* Filter coefficients */
        .include    "bp12k_48.inc"

        .set        N_BP12, N_ALIASING48
        .set        SCALE_BP12, SCALE_ALIASING48

        .bss

        /* Anti-aliasing filter. Required size:
         * (Number of taps = N_BP12) * (2 channels I/Q) * (sizeof(int32) = 4).
         * There's a copy of this ring buffer, so we need twice the size:
         * N_BP12 * 2 * 4 * 2 bytes.
         * Having a copy of the ring buffer allows to always read the complete buffer linearly!
         */
bpfilter32:         .space (N_BP12 * 2 * 4 * 2), 0
        /* Write index into the filter. This is where the next sample goes. */
bpidx:              .word 0


        .text

        .thumb_func
        .global FIFIDSP_bandpass12k_48
FIFIDSP_bandpass12k_48:
        push    {r4-r8,lr}

        /* Get filter write pointer */
        ldr     r12, =bpidx
        ldr     r2, [r12]
        ldr     r3, =bpfilter32
        add     r3, r3, r2, lsl #3

        add     r2, #1                      /* bpidx++ */
        cmp     r2, #N_BP12
        it      ge
        movsge  r2, #0
        str     r2, [r12]

        strd    r0, r1, [r3]                /* Store in ring buffer (filter tap) */
        strd    r0, r1, [r3, #2*4*N_BP12]   /* Store a copy */

        /* Calculate filter */
        add     r3, #(2 * 4)                /* Point to next (=oldest!) sample */
        ldr     r12, =bp12k_48
        mov     r0, #0                      /* sum = 0 */

        .rept N_BP12/2
        ldmia   r3!, {r2, r4, r5, r6}       /* Fetch i/q of two samples */
        ldmia   r12!, {r7, r8}              /* Coefficients */
        mla     r0, r5, r7, r0              /* sum += afilter[i].i * consti[m] */
        mla     r0, r4, r8, r0              /* sum += afilter[i].q * constq[m] */
        .endr

        /* The value in 'sum' contains the sum of i and q branches, and therefore
         * this is the desired sideband as a real signal!
         */

        /* Scale down to 16 significant bits */
        asr     r0, #SCALE_BP12

        pop     {r4-r8,lr}

        bx      lr

        .end
