/* Copyright (c) 2010, DF9DQ
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions
 * and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 * Neither the name of the author nor the names of its contributors may be used to endorse
 * or promote products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdint.h>

#include "fifidsp.h"
#include "ssb2200.h"
#include "ssb2700.h"
#include "ssb3300.h"


extern int FIFIDSP_runSsbFilter (int32_t sx);
extern int32_t ssbFilterSum[2];

#define SSB_DECIM_RATE                      4


static int32_t envn;
static int32_t pif;
static int32_t sxout;



/* Initialize the SSB demodulator */
void FIFIDSP_initSsb (FIFIDSP_AlgoContext *pContext)
{
    pContext->pCoeff = ssbfilter2200;
    if (pContext->bandwidth >= 2500) {
        pContext->pCoeff = ssbfilter2700;
    }
    if (pContext->bandwidth >= 3000) {
        pContext->pCoeff = ssbfilter3300;
    }
//TODO
envn=0;
pif=0;
sxout=0;
}


/* Automatic Gain Control */
static int32_t FIFIDSP_runSsbAgc (FIFIDSP_AlgoContext *pAlgo, int32_t sxin)
{
    int gain;
    int temp;
    int sx;
    (void) pAlgo;


#define AGC_C   0.01
#define AGC_C1  (uint32_t)(AGC_C * 32768.0)
#define AGC_C2  (uint32_t)((1 - AGC_C) * 65536.0 * 65536.0)
#define AGC_PI  2
#define AGC_NOM 5000

    /* abs(sx) */
    sx = sxout;
    if (sx < 0) {
        sx = -sx;
    }

    /* Average filter */
    __asm volatile (
        "umull  %[temp], %[envn], %[envn], %[one_minus_agc]\n\t"
        "add    %[envn], %[envn], %[sx]\n\t"
        : [envn]"+r" (envn), [temp]"=&r" (temp)
        : [sx]"r" (AGC_C1 * sx), [one_minus_agc]"r" (AGC_C2)
    );

    pif = AGC_PI * (AGC_NOM - envn / 32768) + pif;
    pif = __USAT(pif, 22);  /* Max gain = 64 (36 dB) */

    gain = pif;
    sxout = __SSAT((sxin * gain) / 65536, 16);

    return sxout;
}



/* Process one complex input sample of the SSB demodulator */
int FIFIDSP_demodSsb (FIFIDSP_AlgoContext *pContext, int32_t sx, int32_t *pOut)
{
    int polyPhase;
    int32_t temp;


    polyPhase = FIFIDSP_runChannelFilter(sx, pContext->pCoeff);

    /* Check if there's a new sample (last polyphase branch) */
    if (polyPhase == SSB_DECIM_RATE - 1) {

        /* Select sideband */
        __asm volatile (
            "cbz    %[mode], _demodSsb48_usb    \n\t"
            "subs   %[si], %[sq]                \n\t"   /* sumi - sumq (LSB) */
            "b      _demodSsb48_mode_end        \n\t"
        "_demodSsb48_usb:"
            "adds   %[si], %[sq]                \n\t"   /* sumi + sumq (USB) */
        "_demodSsb48_mode_end:"
            "asr    %[sx], %[si], %[SHIFT]      \n\t"

            "movs   %[si], #0                   \n\t"   /* Clear filter accus */
            "movs   %[sq], #0                   \n\t"
            : [sx]"=r" (sx),
              [si]"+r" (channelFilterSum[0]), [sq]"+r" (channelFilterSum[1])
            : [mode]"l" (pContext->mode),
              [SHIFT]"I" (12)
        );

        __asm volatile (
            "lsl    %[sx], %[sx], #1\n\t"
            "smull  %[prodl], %[sx], %[sx], %[x]\n\t"    /* *= gain (0dB = 2^31) */
            : [sx]"+r" (sx), [prodl]"=&r" (temp)
            : [x]"r" (pContext->gain131)
        );

        sx = __SSAT(sx, 16);                            /* Saturate (16 bits) */

        /* Return output sample */
        *pOut = FIFIDSP_runSsbAgc(pContext, sx);

        return 1;
    }

    return 0;
}

