
#include <stdint.h>

#include "bsp-fifisdr.h"

#include "lpclib.h"
#include "task-gui.h"


#define GUI_QUEUE_LENGTH                        (4)


typedef struct {
    uint8_t opcode;
} GUI_Message;


/** Message opcodes for GUI task. */
enum {
    GUI_OPCODE_LED_TICK,
};


/** Identifiers for OS timers. */
enum {
    GUI_TIMERMAGIC_LED,
};


/** Local task context. */
static struct {
    osMailQId queue;
    osTimerId ledTick;
} guiContext;



static void GUI_osalCallback (void const *pArgument)
{
    (void) pArgument;

#if 0
    GUI_Message message;

    message.opcode = GUI_OPCODE_LED_TICK;
    osMailPut(guiContext.queue, &message);
#else
    GUI_Message *pMessage = osMailAlloc(guiContext.queue, 0);

    if (pMessage == NULL) {
        return;
    }

    pMessage->opcode = GUI_OPCODE_LED_TICK;
    osMailPut(guiContext.queue, pMessage);
#endif
}


osTimerDef(led, GUI_osalCallback);



void GUI_task (const void *pArgs)
{
    (void) pArgs;
    GUI_Message *pMessage;
    osEvent event;


    guiContext.ledTick = osTimerCreate(osTimer(led), osTimerPeriodic, (void *)GUI_TIMERMAGIC_LED);
    osTimerStart(guiContext.ledTick, 500);

    while (1) {
        /* Is there a new message? */
        event = osMailGet(guiContext.queue, osWaitForever);
        if (event.status == osEventMail) {
            pMessage = (GUI_Message *)event.value.p;
            switch (pMessage->opcode) {
            case GUI_OPCODE_LED_TICK:
                GPIO_writeBit(LED, GPIO_readBit(LED) ^ 1);
                break;
            }

            osMailFree(guiContext.queue, event.value.p);
        }
    }
}


osMailQDef(guiQueue, GUI_QUEUE_LENGTH, GUI_Message);


/** GUI module init function.
 *
 *  Must be called before the scheduler starts. Initializes the message queue.
 */
void GUI_initModule (void)
{
    /* Task communication */
    guiContext.queue = osMailCreate(osMailQ(guiQueue), NULL);
}

