/* Copyright (c) 2010, DF9DQ
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions
 * and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 * Neither the name of the author nor the names of its contributors may be used to endorse
 * or promote products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



#include <stdio.h>
#include <string.h>

#include "lpclib.h"
#include "bsp-fifisdr.h"

#include "usbuser.h"
#include "softrock.h"



const uint8_t audioEndpointList1[] = {USBCONFIG_STREAMING_EP1};
const uint8_t audioEndpointList2[] = {USBCONFIG_STREAMING_EP2};



/*************** Audio Class **********************/

/** Describe the range for all audio controls. */
#define FIFISDR_AUDIO_NUM_CONTROLS_1        2
#define FIFISDR_AUDIO_NUM_CONTROLS_2        3

static USBAUDIO_ControlRange audioControls1[FIFISDR_AUDIO_NUM_CONTROLS_1] = {
    { .unit_id          = USBCONFIG_UNIT_FEATURE_IQ,
      .control_id       = USBAC_CS_FU_MUTE_CONTROL,
      .min              = 0,
      .max              = 1,
      .current          = 0,
      .resolution       = 1,
      .length           = 1,
      .callback         = NULL,
    },
    { .unit_id          = USBCONFIG_UNIT_FEATURE_IQ,
      .control_id       = USBAC_CS_FU_VOLUME_CONTROL,
      .min              = -6 * 0x100,
      .max              = 0,
      .current          = 0,
      .resolution       = 6 * 0x100,
      .length           = 2,
      .callback         = FIFIAUDIO_handleEventIQ,
    },
};

static USBAUDIO_ControlRange audioControls2[FIFISDR_AUDIO_NUM_CONTROLS_2] = {
    { .unit_id          = USBCONFIG_UNIT_FEATURE_DSP,
      .control_id       = USBAC_CS_FU_MUTE_CONTROL,
      .min              = 0,
      .max              = 1,
      .current          = 0,
      .resolution       = 1,
      .length           = 1,
      .callback         = FIFIAUDIO_handleEventDemod,
    },
    { .unit_id          = USBCONFIG_UNIT_FEATURE_DSP,
      .control_id       = USBAC_CS_FU_VOLUME_CONTROL,
      .min              = -60 * 0x100,
      .max              = 0,
      .current          = 0,
      .resolution       = 1 * 0x100,
      .length           = 2,
      .callback         = FIFIAUDIO_handleEventDemod,
    },
    { .unit_id          = USBCONFIG_UNIT_FEATURE_DSP,
      .control_id       = USBAC_CS_FU_AUTOMATIC_GAIN_CONTROL,
      .min              = 0,
      .max              = 1,
      .current          = 1,
      .resolution       = 1,
      .length           = 1,
      .callback         = FIFIAUDIO_handleEventDemod,
    },
};


const USBAUDIO_ControlInterface fifisdr_audioControlInterface1 = {
    .interfaceNumber            = USBCONFIG_INTERFACE_AUDIO_CONTROL_1,
    .usesInterrupt              = false,
    .endpointNumberInterrupt    = 0,
    .numControls                = FIFISDR_AUDIO_NUM_CONTROLS_1,
    .controls                   = audioControls1,
};


const USBAUDIO_ControlInterface fifisdr_audioControlInterface2 = {
    .interfaceNumber            = USBCONFIG_INTERFACE_AUDIO_DSP_CONTROL,
    .usesInterrupt              = false,
    .endpointNumberInterrupt    = 0,
    .numControls                = FIFISDR_AUDIO_NUM_CONTROLS_2,
    .controls                   = audioControls2,
};

#if BOARD_TYPE == BOARD_ROLF2
const uint32_t fifisdr_samplingFrequencies1_16[] = {48000ul, 96000ul, 192000ul};
#else
const uint32_t fifisdr_samplingFrequencies1_16[] = {48000ul, 96000ul};
#endif
const uint32_t fifisdr_samplingFrequencies1_32[] = {48000ul, 96000ul};

const USBAUDIO_StreamingInterfaceParams fifisdr_streamingParams1[] = {
    { .numEndpoints             = 0,
    },
    { .numEndpoints             = 1,
      .linkedTerminal           = USBCONFIG_UNIT_TERMINAL_OUT_IQ,
      .endpointNumber           = USBCONFIG_STREAMING_EP1,
      .packetSize               = USBCONFIG_ISOC_SIZE1_32,
      .supportedControls        = 0x01,     /* Sampling frequency */
      .numSamplingFrequencies   = 2,
      .samplingFrequencies      = fifisdr_samplingFrequencies1_32,
    },
    { .numEndpoints             = 1,
      .linkedTerminal           = USBCONFIG_UNIT_TERMINAL_OUT_IQ,
      .endpointNumber           = USBCONFIG_STREAMING_EP1,
      .packetSize               = USBCONFIG_ISOC_SIZE1_16,
      .supportedControls        = 0x01,     /* Sampling frequency */
#if BOARD_TYPE == BOARD_ROLF2
      .numSamplingFrequencies   = 3,
#else
      .numSamplingFrequencies   = 2,
#endif
      .samplingFrequencies      = fifisdr_samplingFrequencies1_16,
    },
};


const uint32_t fifisdr_samplingFrequencies2[] = {12000ul};

const USBAUDIO_StreamingInterfaceParams fifisdr_streamingParams2[] = {
    { .numEndpoints             = 0,
    },
    { .numEndpoints             = 1,
      .linkedTerminal           = USBCONFIG_UNIT_TERMINAL_OUT_DSP,
      .endpointNumber           = USBCONFIG_STREAMING_EP2,
      .packetSize               = USBCONFIG_ISOC_SIZE2,
      .supportedControls        = 0x00,
      .numSamplingFrequencies   = 1,
      .samplingFrequencies      = fifisdr_samplingFrequencies2,
    },
};


USBAUDIO_StreamingInterface fifisdr_streamingInterfaces1[] = {
    { .interfaceNumber          = USBCONFIG_INTERFACE_AUDIO_STREAMING_1,
      .numAltSettings           = 3,
      .activeSetting            = 0,
      .currentSamplerate        = 1,
      .isMuted                  = false,
      .params                   = fifisdr_streamingParams1,
    },
};

USBAUDIO_StreamingInterface fifisdr_streamingInterfaces2[] = {
    { .interfaceNumber          = USBCONFIG_INTERFACE_AUDIO_DSP_STREAMING,
      .numAltSettings           = 2,
      .activeSetting            = 0,
      .currentSamplerate        = 1,
      .isMuted                  = false,
      .params                   = fifisdr_streamingParams2,
    },
};


static const USBAUDIO_FunctionDeclaration fifisdr_audioFunctionIQ = {
    .controlInterface       = &fifisdr_audioControlInterface1,
    .numStreamingInterfaces = 1,
    .streamingInterfaces    = fifisdr_streamingInterfaces1,
    .callback               = FIFIAUDIO_handleEventIQ,
};

static const USBAUDIO_FunctionDeclaration fifisdr_audioFunctionDemod = {
    .controlInterface       = &fifisdr_audioControlInterface2,
    .numStreamingInterfaces = 1,
    .streamingInterfaces    = fifisdr_streamingInterfaces2,
    .callback               = FIFIAUDIO_handleEventDemod,
};


static USBAUDIO_Handle handleAudioIQ;
#if USBCONFIG_DSP
static USBAUDIO_Handle handleAudioDemod;
#endif


static const USB_Class fifisdr_classes[] = {
    {
      .configuration    = 1,
      .numEndpoints     = 0,
      .pEndpointList    = NULL,
      .init             = NULL,
      .requestHandler   = softrock_handleRequests,
      .frameHandler     = NULL,
      .endpointHandler  = NULL,
      .pInstance        = NULL,
    },
#if USBCONFIG_SOUNDCARD
    {
      .configuration    = 1,
      .numEndpoints     = 1,
      .pEndpointList    = audioEndpointList1,
      .init             = NULL,
      .requestHandler   = USBAUDIO_handleRequests,
      .frameHandler     = FIFIAUDIO_frameHandler,
      .endpointHandler  = usbaudio_handleEndpointData,
      .pInstance        = &handleAudioIQ,
    },
  #if USBCONFIG_DSP
    {
      .configuration    = 1,
      .numEndpoints     = 1,
      .pEndpointList    = audioEndpointList2,
      .init             = NULL,
      .requestHandler   = USBAUDIO_handleRequests,
      .frameHandler     = NULL,
      .endpointHandler  = usbaudio_handleEndpointData,
      .pInstance        = &handleAudioDemod,
    },
  #endif
#endif
};
const uint8_t fifisdr_numClasses = sizeof(fifisdr_classes) / sizeof(fifisdr_classes[0]);



/* Definition of USB task with restricted resource access (MPU) */
//#define USB_TASK_STACKSIZE      (1024)


USBDEV_Handle usbdev;


/** USB handler task.
 */
void USBUSER_task (const void *pArgs)
{
    (void) pArgs;
I2S_Handle i2sHandle;

    USBDEV_Config config;


    fifisdr_initDescriptors();

    /* Prepare the classes. */
#if USBCONFIG_SOUNDCARD
    USBAUDIO_open(&fifisdr_audioFunctionIQ, &handleAudioIQ);
  #if USBCONFIG_DSP
    USBAUDIO_open(&fifisdr_audioFunctionDemod, &handleAudioDemod);
  #endif
#endif

//TODO
I2S_open(I2S0, &i2sHandle);

NVIC_EnableIRQ(CAN_IRQn);
NVIC_SetPriority(CAN_IRQn, 1);
NVIC_EnableIRQ(CANActivity_IRQn);
NVIC_SetPriority(CANActivity_IRQn, 1);

    USBDEV_open(usbHardware, &usbdev);

    config.opcode = USBDEV_OPCODE_ATTACH_CLASSES,
    config.count = fifisdr_numClasses,
    config.pClassList = (USB_Class *)fifisdr_classes,
    USBDEV_ioctl(usbdev, &config);                      /* Attach class descriptors */

    config.opcode = USBDEV_OPCODE_ATTACH_DESCRIPTORS,
    config.pDescriptors = (USB_DescriptorList *)&fifisdr_descriptors,
    USBDEV_ioctl(usbdev, &config);

config.opcode = USBDEV_OPCODE_CONNECT;
USBDEV_ioctl(usbdev, &config);    //TODO

    while (1) {
        USBDEV_worker(usbdev);
    }

}

//TODO
void CAN_IRQHandler (void)
{
}

void CANActivity_IRQHandler (void)
{
}



/** Module init function.
 *
 *  Must be called before the scheduler starts. Initializes the message queue.
 */
void USBUSER_initModule (void)
{
    /* Task communication */
//    guiQueue = xQueueCreate((unsigned portBASE_TYPE) GUI_QUEUE_LENGTH,
//                            (unsigned portBASE_TYPE) sizeof(GUI_Message));
}

