
#include <stdint.h>


/* External functions */
extern int main (void);
extern void SystemInit (void);

/* These section descriptors are defined in the linker script */
extern uint32_t __stack_end__;
extern uint32_t __bss_start__;
extern uint32_t __bss_end__;
extern uint32_t __bss2_start__;
extern uint32_t __bss2_end__;
extern uint32_t __bss3_start__;
extern uint32_t __bss3_end__;
extern uint32_t __data_start__;
extern uint32_t __data_end__;
extern uint32_t __data2_start__;
extern uint32_t __data2_end__;
extern uint32_t __data3_start__;
extern uint32_t __data3_end__;
extern uint32_t __end_of_text__;



/* Default interrupt service routines.
 * Defined as weak symbols. Can be redefined in the application.
 */
#define ALIAS(def) __attribute__((weak,alias(#def)))

void Default_IRQHandler( void );

void Reset_Handler( void );
void NMI_Handler( void );
void HardFault_Handler( void );
void MPU_Handler( void );
void BusFault_Handler( void );
void UsageFault_Handler( void );

extern void vPortSVCHandler (void);
extern void xPortPendSVHandler (void);
extern void xPortSysTickHandler (void);


void WDT_IRQHandler( void ) ALIAS(Default_IRQHandler);
void Timer0_IRQHandler( void ) ALIAS(Default_IRQHandler);
void Timer1_IRQHandler( void ) ALIAS(Default_IRQHandler);
void Timer2_IRQHandler( void ) ALIAS(Default_IRQHandler);
void Timer3_IRQHandler( void ) ALIAS(Default_IRQHandler);
void UART0_IRQHandler( void ) ALIAS(Default_IRQHandler);
void UART1_IRQHandler( void ) ALIAS(Default_IRQHandler);
void UART2_IRQHandler( void ) ALIAS(Default_IRQHandler);
void UART3_IRQHandler( void ) ALIAS(Default_IRQHandler);
void PWM1_IRQHandler( void ) ALIAS(Default_IRQHandler);
void I2C0_IRQHandler( void ) ALIAS(Default_IRQHandler);
void I2C1_IRQHandler( void ) ALIAS(Default_IRQHandler);
void I2C2_IRQHandler( void ) ALIAS(Default_IRQHandler);
void SPI_IRQHandler( void ) ALIAS(Default_IRQHandler);
void SSP0_IRQHandler( void ) ALIAS(Default_IRQHandler);
void SSP1_IRQHandler( void ) ALIAS(Default_IRQHandler);
void PLL0_IRQHandler( void ) ALIAS(Default_IRQHandler);
void RTC_IRQHandler( void ) ALIAS(Default_IRQHandler);
void EINT0_IRQHandler( void ) ALIAS(Default_IRQHandler);
void EINT1_IRQHandler( void ) ALIAS(Default_IRQHandler);
void EINT2_IRQHandler( void ) ALIAS(Default_IRQHandler);
void EINT3_IRQHandler( void ) ALIAS(Default_IRQHandler);
void ADC_IRQHandler( void ) ALIAS(Default_IRQHandler);
void BOD_IRQHandler( void ) ALIAS(Default_IRQHandler);
void USB_IRQHandler( void ) ALIAS(Default_IRQHandler);
void CAN_IRQHandler( void ) ALIAS(Default_IRQHandler);
void DMA_IRQHandler( void ) ALIAS(Default_IRQHandler);
void I2S_IRQHandler( void ) ALIAS(Default_IRQHandler);
void Ethernet_IRQHandler( void ) ALIAS(Default_IRQHandler);
void RIT_IRQHandler( void ) ALIAS(Default_IRQHandler);
void MCPWM_IRQHandler( void ) ALIAS(Default_IRQHandler);
void QEI_IRQHandler( void ) ALIAS(Default_IRQHandler);
void PLL1_IRQHandler( void ) ALIAS(Default_IRQHandler);
void USBActivity_IRQHandler( void ) ALIAS(Default_IRQHandler);
void CANActivity_IRQHandler( void ) ALIAS(Default_IRQHandler);



/**
 * Vector table
 */
void (*vector_table[])( void ) __attribute__ ((section(".vectors"))) =
{
    (void(*)(void))&__stack_end__,
    Reset_Handler,
    NMI_Handler,
    HardFault_Handler,
    MPU_Handler,
    BusFault_Handler,
    UsageFault_Handler,
    0,
    0,
    0,
    0,
    vPortSVCHandler,
    0,
    0,
    xPortPendSVHandler,
    xPortSysTickHandler,

    WDT_IRQHandler,
    Timer0_IRQHandler,
    Timer1_IRQHandler,
    Timer2_IRQHandler,
    Timer3_IRQHandler,
    UART0_IRQHandler,
    UART1_IRQHandler,
    UART2_IRQHandler,
    UART3_IRQHandler,
    PWM1_IRQHandler,
    I2C0_IRQHandler,
    I2C1_IRQHandler,
    I2C2_IRQHandler,
    SPI_IRQHandler,
    SSP0_IRQHandler,
    SSP1_IRQHandler,

    PLL0_IRQHandler,
    RTC_IRQHandler,
    EINT0_IRQHandler,
    EINT1_IRQHandler,
    EINT2_IRQHandler,
    EINT3_IRQHandler,
    ADC_IRQHandler,
    BOD_IRQHandler,
    USB_IRQHandler,
    CAN_IRQHandler,
    DMA_IRQHandler,
    I2S_IRQHandler,
    Ethernet_IRQHandler,
    RIT_IRQHandler,
    MCPWM_IRQHandler,
    QEI_IRQHandler,
    PLL1_IRQHandler,
    USBActivity_IRQHandler,
    CANActivity_IRQHandler,
};




uint32_t *pRam __attribute__((section(".pRam")));
uint32_t *pRom;



void Reset_Handler( void )
{
    /* Init the clock tree */
    SystemInit();

    /* Initialize data segments with the initializers from flash.
     * data is in the 'main' RAM block, starting at address 0x10000000.
     * data2 and data3 are located in the two other 16 KiB RAM blocks
     * at 0x2007C000 and 0x20080000 (64 KiB parts).
     */
    pRom = (uint32_t *)&__end_of_text__;
    for (pRam = &__data_start__; pRam < &__data_end__; pRam++) {
        *pRam = *pRom++;
    }
    for (pRam = &__data2_start__; pRam < &__data2_end__; pRam++) {
        *pRam = *pRom++;
    }
    for (pRam = &__data3_start__; pRam < &__data3_end__; pRam++) {
        *pRam = *pRom++;
    }

    /* Clear the bss sections in RAM.
     * bss is in the 'main' RAM block, starting at address 0x10000000.
     * bss2 and bss3 are located in the two other 16 KiB RAM blocks
     * at 0x2007C000 and 0x20080000 (64 KiB parts).
     */
    for (pRam = &__bss_start__; pRam < &__bss_end__; pRam++) {
        *pRam = 0;
    }
    for (pRam = &__bss2_start__; pRam < &__bss2_end__; pRam++) {
        *pRam = 0;
    }
    for (pRam = &__bss3_start__; pRam < &__bss3_end__; pRam++) {
        *pRam = 0;
    }

    /* Now we can jump to the application entry point main() */
    main();

    /* Loop here forever, in case main() ever returns. */
    while (1) {
    }
}



/*------ Core exception handlers --------------------------------------------*/

void NMI_Handler (void)
{
    while (1) {
    }
}


void HardFault_Handler (void)
{
    while (1) {
    }
}


void MPU_Handler (void)
{
    while (1) {
    }
}


void BusFault_Handler (void)
{
    while (1) {
    }
}


void UsageFault_Handler (void)
{
    while (1) {
    }
}


/*------ Default IRQ handler ------------------------------------------------*/

void Default_IRQHandler (void)
{
    while (1) {
    }
}

