/* $Id::                                                               $
 *
 * Copyright (c) 2011, NXP Semiconductors
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list
 * of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list
 * of conditions and the following disclaimer in the documentation and/or
 * other materials provided with the distribution.
 * Neither the name of the author nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "lpc17xx_libconfig.h"


/** \addtogroup GPIO
 *  @{
 */

#include "lpc17xx_clkpwr.h"
#include "lpc17xx_gpio.h"



osMutexId gpioMutex;
osMutexDef(gpioMutexDef);



#if LPCLIB_GPIO_INTERRUPTS
static LPCLIB_Callback gpioCallback;
#endif


void GPIO_open (void)
{
    CLKPWR_enableClock(CLKPWR_CLOCKSWITCH_GPIO);
    gpioMutex = osMutexCreate(osMutex(gpioMutexDef));
}


void GPIO_setDir32 (GPIO_Port32 port, uint32_t value, uint32_t mask)
{
    if (osMutexWait(gpioMutex, osWaitForever) == osOK) {
        __gpio[port & 3]->FIODIR32 = (__gpio[port & 3]->FIODIR32 & ~(~value & mask)) | (value & mask);

        osMutexRelease(gpioMutex);
    }
}

void GPIO_setDirBit (GPIO_Pin pin, LPCLIB_Switch outputEnable)
{
    if (osMutexWait(gpioMutex, osWaitForever) == osOK) {
        __gpio[(pin >> 0) & 7]->FIODIR32 = (__gpio[(pin >> 0) & 7]->FIODIR32 & ~(1u << ((pin >> 3) & 0x1F))) | (outputEnable << ((pin >> 3) & 0x1F));

        osMutexRelease(gpioMutex);
    }
}


/* Set GPIO options. */
void GPIO_ioctl (const GPIO_Config *pConfig)
{
#if LPCLIB_GPIO_INTERRUPTS
    int port;
    LPC_GPIO_TypeDef *gpio;
    uint32_t mask;
#endif


    switch (pConfig->opcode) {
#if LPCLIB_GPIO_INTERRUPTS
    case GPIO_OPCODE_SET_CALLBACK:
        gpioCallback = pConfig->callbackFromISR;
        break;

    case GPIO_OPCODE_CONFIGURE_INTERRUPT:
        gpio = __gpio[(pConfig->interrupt.pin >> 0) & 3];
        mask = 1u << ((pConfig->interrupt.pin >> 3) & 0x0F);

        gpio->IE &= ~mask;                              /* Always mask interrupts during configuration
                                                         * to avoid glitches
                                                         */
        if (pConfig->interrupt.enable) {
            switch (pConfig->interrupt.mode) {
            case GPIO_INT_FALLING_EDGE:
                gpio->IS &= ~mask;                      /* Edge sensitive */
                gpio->IEV &= ~mask;                     /* Falling edge */
                gpio->IBE &= ~mask;                     /* Single edge */
                break;
            case GPIO_INT_RISING_EDGE:
                gpio->IS &= ~mask;                      /* Edge sensitive */
                gpio->IEV |= mask;                      /* Rising edge level */
                gpio->IBE &= ~mask;                     /* Single edge */
                break;
            case GPIO_INT_BOTH_EDGES:
                gpio->IS &= ~mask;                      /* Edge sensitive */
                gpio->IBE |= mask;                      /* Both edges */
                break;
            }

            gpio->IC = mask;                            /* Prime edge detector */
            gpio->IE |= mask;                           /* Now unmask interrupts */
        }
        break;
#endif

    default:
        break;
    }
}



#if LPCLIB_GPIO_INTERRUPTS

void EINT3_IRQHandler (void)
{
#if 1
    LPCLIB_Event event;

    uint32_t status = __gpio[port]->MIS;                /* Read interrupt sources */
    __gpio[port]->IC = status;                          /* Clear the requests */

    if (gpioCallback) {
        event.id = LPCLIB_EVENTID_GPIO;
        event.block = port;
        event.parameter = (void *)status;
        gpioCallback(event);
    }
#endif
}

#endif

/** @} */

